'use strict';

var compact = require('lodash/compact');
var flatten = require('lodash/flatten');

var helpers = require('../../helpers');
var stripHtml = helpers.stripHtml;
var currentYear = helpers.currentYear();

var CONSTANTS = require('./constants');
var ASSET_TYPES = CONSTANTS.ASSET_TYPES;
var ASSET_TYPE_VALUES = CONSTANTS.ASSET_TYPE_VALUES;
var NODE_TYPES = CONSTANTS.NODE_TYPES;
var PERMITTED_TYPES = CONSTANTS.PERMITTED_TYPES;
var WEB_LINK_RESOURCES = CONSTANTS.WEB_LINK_RESOURCES;
var BASIC_LTI_RESOURCES = CONSTANTS.BASIC_LTI_RESOURCES;
var LABEL_BLACKLIST = CONSTANTS.LABEL_BLACKLIST;
var NOOP_ARRAY = CONSTANTS.NOOP_ARRAY;
var LAUNCHABLE_TYPES = CONSTANTS.LAUNCHABLE_TYPES;
var LAUNCHABLE_TYPE_GROUPS = CONSTANTS.LAUNCHABLE_TYPE_GROUPS;
var CARTRIDGE_TYPES = CONSTANTS.CARTRIDGE_TYPES;
var BASE_LURE_URL = CONSTANTS.BASE_LURE_URL;
var COURSEWARE_WHITELIST_KEYS = CONSTANTS.COURSEWARE_WHITELIST_KEYS;
var BENTO_WHITELIST_KEYS = CONSTANTS.BENTO_WHITELIST_KEYS;
var ASSET_TYPE_ALIASES = CONSTANTS.ASSET_TYPE_ALIASES;
var LANGUAGE_CODE_MAP = CONSTANTS.LANGUAGE_CODE_MAP;
var COUNTRY_CODE_MAP = CONSTANTS.COUNTRY_CODE_MAP;
var LURE_SERVICES = CONSTANTS.LURE_SERVICES;

var _ = {
  getType: function getType(item) {
    return item.nodeType || item.assetType;
  },
  getAssetTypeAlias: function getAssetTypeAlias(item) {
    if (!_.isAsset(item)) {
      return;
    }

    var alias = ASSET_TYPE_ALIASES[item.assetType];

    if (!alias) {
      return;
    }

    return alias;
  },
  getCartridgeType: function getCartridgeType(item) {
    if (_.isWebLink(item)) {
      return CARTRIDGE_TYPES.WEB_LINK;
    }
    if (_.isBasicLTILink(item)) {
      return CARTRIDGE_TYPES.BASIC_LTI_LINK;
    }
  },
  hasParent: function hasParent(item) {
    return !!item._parent;
  },
  isAssessment: function isAssessment(item) {
    var type = _.getType(item);
    return [ASSET_TYPES.ASSESSMENT, ASSET_TYPES.BANK_ASSESSMENT].indexOf(type) !== -1;
  },
  isAsset: function isAsset(item) {
    return !!item.assetType;
  },
  isNode: function isNode(item) {
    return !!item.nodeType;
  },
  isResource: function isResource(item) {
    if (!item.assetType) {
      return false;
    }

    return ASSET_TYPE_VALUES.indexOf(item.assetType) !== -1;
  },
  isAddOn: function isAddOn(item) {
    return item.nodeType === NODE_TYPES.ADD_ON;
  },
  isPermitted: function isPermitted(item, whitelist) {
    return (PERMITTED_TYPES.indexOf(_.getType(item)) !== -1 || _.isPermittedSubsection(item, whitelist)) && _.isWhitelisted(item, whitelist);
  },
  isWhitelisted: function isWhitelisted(item, whitelist) {
    if (!whitelist) {
      return true;
    }

    var benchmarks = _.benchmarks(item);

    if (!benchmarks) {
      return true;
    }

    return whitelist.map(function (a) {
      return benchmarks.map(function (b) {
        return b[BENTO_WHITELIST_KEYS.COURSE].secondaryId === a[COURSEWARE_WHITELIST_KEYS.COURSE] && b[BENTO_WHITELIST_KEYS.DOCUMENT].secondaryId === a[COURSEWARE_WHITELIST_KEYS.DOCUMENT] && b[BENTO_WHITELIST_KEYS.AUTHORITY].secondaryId === a[COURSEWARE_WHITELIST_KEYS.AUTHORITY] && b[BENTO_WHITELIST_KEYS.SUBJECT_DOCUMENT].secondaryId === a[COURSEWARE_WHITELIST_KEYS.SUBJECT_DOCUMENT];
      }).indexOf(true) !== -1;
    }).indexOf(true) !== -1;
  },
  isParentAddOn: function isParentAddOn(item) {
    return _.hasParent(item) && _.isAddOn(item._parent);
  },
  isPermittedSubsection: function isPermittedSubsection(item, whitelist) {
    return _.isNode(item) && item.nodeType === NODE_TYPES.SUBSECTION && _.hasChildren(item, whitelist);
  },
  isWebLink: function isWebLink(item) {
    return WEB_LINK_RESOURCES.indexOf(item.assetType) !== -1;
  },
  isBasicLTILink: function isBasicLTILink(item) {
    return BASIC_LTI_RESOURCES.indexOf(item.assetType) !== -1;
  },
  isBlacklistedLabel: function isBlacklistedLabel(item) {
    if (!_.isNode(item)) {
      return false;
    }
    return LABEL_BLACKLIST.indexOf(item.nodeType) !== -1;
  },
  isNoopArray: function isNoopArray(arr) {
    return arr === NOOP_ARRAY;
  },
  isSubsection: function isSubsection(item) {
    return item.nodeType === NODE_TYPES.SUBSECTION;
  },
  isTeacherPresentation: function isTeacherPresentation(item) {
    var displayOptions = item.displayOptions;

    if (!displayOptions) {
      return;
    }

    var ngc = displayOptions.ngc;

    if (!ngc) {
      return;
    }

    return !!ngc.presentationLearnerVisible;
  },
  isStudentSubmissionRequired: function isStudentSubmissionRequired(item) {
    return !!item.isSubmissionRequired;
  },
  hasChildren: function hasChildren(item, whitelist) {
    if (!item.children) {
      return false;
    }

    var children = item.children.filter(function (item) {
      return _.isPermitted(item, whitelist);
    });

    return children.length > 0;
  },
  title: function title(item) {
    var title = stripHtml(item.title) || _.findMetadata('displayTitle', item)[0].text;
    var label = void 0;
    if (!_.isBlacklistedLabel(item)) {
      label = item.label;
    }

    return compact([label, title]).join(': ') || _.getType(item);
  },
  url: function url(item) {
    return item.assetRef;
  },
  year: function year(dateTime) {
    if (!dateTime) {
      return;
    }

    return String(new Date(dateTime).getFullYear());
  },
  lureDomain: function lureDomain(env) {
    return BASE_LURE_URL(env);
  },
  courseXid: function courseXid(item) {
    if (item.nodeType !== NODE_TYPES.COURSE) {
      return;
    }

    var id = item.id,
        xid = item.xid,
        publishId = item.publishId;


    if (!id || !xid || !publishId) {
      return;
    }

    // urn:com.mheducation.openlearning:cms.authoring:{sdlc}.cms:nodes:{courseId}:revision 1"
    var segments = xid.split(':');

    // urn:com.mheducation.openlearning:cms.authoring:{sdlc}.cms:courseware:{courseId}:{publishId}
    return [segments[0], segments[1], segments[2], segments[3], 'courseware', id, publishId].join(':');
  },
  description: function description(item) {
    return _.findMetadata('desc', item)[0].text || stripHtml(item.title);
  },
  convertGrade: function convertGrade(value) {
    var grade = void 0;

    if (!value) {
      return;
    }

    grade = parseInt(value, 10);

    if (isNaN(grade)) {
      grade = value.charAt(0).toLowerCase();
    }

    if (['p', 'k'].indexOf(grade) !== -1) {
      return grade;
    }

    // Adult
    if (grade === 'h' || grade > 12) {
      return 'a';
    }

    return grade;
  },
  gradeRange: function gradeRange(item) {
    var from = _.convertGrade(_.findMetadata('gradeMin', item)[0].text);
    var to = _.convertGrade(_.findMetadata('gradeMax', item)[0].text);

    if (from === to) {
      to = null;
    }

    return compact([from, to]).join(' to ');
  },
  keywords: function keywords(item) {
    var metadata = _.findMetadata('keyword', item);
    var keywords = compact(flatten([_.subject(item), _.typeGroup(item), _.resourceType(item), _.assetType(item), metadata.map(function (k) {
      return k.text;
    })]));
    return Array.from(new Set(keywords));
  },
  benchmarks: function benchmarks(item) {
    var list = _.findMetadata('benchmark', item);

    if (_.isNoopArray(list)) {
      return;
    }

    return list;
  },
  resourceType: function resourceType(item) {
    return _.getAssetTypeAlias(item);
  },
  assetType: function assetType(item) {
    return _.findMetadata('type', item)[0].text;
  },
  copyrightOrg: function copyrightOrg(item) {
    return _.findMetadata('copyrightOrg', item)[0].text;
  },
  copyrightYear: function copyrightYear(item) {
    return _.findMetadata('copyrightYear', item)[0].text || _.year(item.publishedTime);
  },
  copyright: function copyright(item) {
    var entity = _.copyrightOrg(item) || _.author(item);
    var date = _.copyrightYear(item) || currentYear;
    var rights = _.rights(item);
    var credit = _.credit(item);

    if (!entity) {
      return;
    }

    return {
      entity: entity,
      date: date,
      rights: rights,
      credit: credit
    };
  },
  author: function author(item) {
    return _.findMetadata('authorOrg', item)[0].text;
  },
  rights: function rights(item) {
    return _.findMetadata('rightsCategory', item)[0].text;
  },
  credit: function credit(item) {
    return _.findMetadata('creditDisplay', item)[0].text;
  },
  language: function language(item) {
    return _.findMetadata('language', item)[0].text;
  },
  country: function country(item) {
    return _.findMetadata('country', item)[0].text;
  },
  languageISO: function languageISO(item) {
    var language = LANGUAGE_CODE_MAP[_.language(item)];
    var country = COUNTRY_CODE_MAP[_.country(item)];

    if (!language) {
      return;
    }

    return compact([language, country]).join('-');
  },
  subject: function subject(item) {
    return _.findMetadata('subject', item)[0].text;
  },
  typeGroup: function typeGroup(item) {
    return _.findMetadata('typeGroup', item)[0].text;
  },
  assetXid: function assetXid(item) {
    return _.findMetadata('MHEAssessLarsAssetXID', item)[0].text || item.larsXid || item.ziraXid || item.xid;
  },
  assetPlayerType: function assetPlayerType(item) {
    var type = _.getType(item);

    if (!type) {
      return;
    }

    /**
     * INTERACTIVE
     */
    if (LAUNCHABLE_TYPE_GROUPS.INTERACTIVE.indexOf(type) !== -1) {
      return LAUNCHABLE_TYPES.INTERACTIVE;
    }

    /**
     * BASIC
     */
    if (LAUNCHABLE_TYPE_GROUPS.BASIC.indexOf(type) !== -1) {
      return LAUNCHABLE_TYPES.BASIC;
    }

    /**
     * STATIC
     */
    if (LAUNCHABLE_TYPE_GROUPS.STATIC.AUDIO.indexOf(type) !== -1) {
      return LAUNCHABLE_TYPES.STATIC.AUDIO;
    }
    if (LAUNCHABLE_TYPE_GROUPS.STATIC.IMAGE.indexOf(type) !== -1) {
      return LAUNCHABLE_TYPES.STATIC.IMAGE;
    }
    if (LAUNCHABLE_TYPE_GROUPS.STATIC.VIDEO.indexOf(type) !== -1) {
      return LAUNCHABLE_TYPES.STATIC.VIDEO;
    }

    // Catch-all unhandled types.
    return LAUNCHABLE_TYPES.BASIC;
  },
  assetServiceType: function assetServiceType(item) {
    if (item.larsXid || item.xid && item.xid.indexOf('lars_asset') !== -1) {
      return LURE_SERVICES.LARS;
    }
    if (item.ziraXid || item.assetRef) {
      return LURE_SERVICES.ZIRA;
    }
  },
  /**
   * Order assets before nodes, and some nodes before other nodes, but preserve position otherwise.
   */
  orderByType: function orderByType(itemA, itemB) {
    // Order assets
    if (_.isAsset(itemA) && _.isNode(itemB)) {
      return -1;
    }
    if (_.isNode(itemA) && _.isAsset(itemB)) {
      return 1;
    }

    // Order nodes
    if (_.isNode(itemA) && _.isNode(itemB)) {
      // Sections
      if (itemA.nodeType === NODE_TYPES.SECTION && itemB.nodeType !== NODE_TYPES.SECTION) {
        return -1;
      }
      if (itemA.nodeType !== NODE_TYPES.SECTION && itemB.nodeType === NODE_TYPES.SECTION) {
        return 1;
      }
    }

    return itemA.sequence - itemB.sequence;
  },
  roles: function roles(item) {
    var intendedAudience = _.findMetadata('intendedAudience', item)[0].text;
    var roles = [intendedAudience];

    if (!item.isLearnerHidden) {
      roles.push('Learner');
    }

    if (item.isLearnerHidden || item.isInstructorOnly) {
      roles.push('Instructor');
    }

    // Remove duplicate values.
    roles = Array.from(new Set(roles));

    return compact(roles);
  },
  // LOM: parent | student | teacher | teacher/uploader | school admin | principal | curriculum admin | admin
  accessPermission: function accessPermission(item) {
    // Order by highest role last.
    var ROLES = ['student', 'parent', 'teacher', 'admin'];
    var roles = _.roles(item).map(function (role) {
      switch (role) {
        case 'Administrator':
          return ROLES.indexOf('admin'); // 3
        case 'Instructor':
          return ROLES.indexOf('teacher'); // 2
        case 'Guardian':
          return ROLES.indexOf('parent'); // 1
        default:
          return ROLES.indexOf('student'); // 0
      }
    });
    var max = roles.reduce(function (a, b) {
      return Math.max(a, b);
    });

    return ROLES[max];
  },
  // Assembly: Learner | Instructor | Guardian | Administrator
  // LOM: Learner | Instructor | Mentor
  intendedEndUserRoles: function intendedEndUserRoles(item, isV1P2Manifest) {
    if (isV1P2Manifest) {
      return _.v1p2ManifestIntendedEndUserRoles(item);
    }

    return _.roles(item).filter(function (a) {
      return ['Learner', 'Instructor'].indexOf(a) !== -1;
    });
  },
  // LOM (v1p2): higher education | school | training | other
  v1p2ManifestIntendedEndUserRoles: function v1p2ManifestIntendedEndUserRoles(item) {
    var from = _.convertGrade(_.findMetadata('gradeMin', item)[0].text);
    var to = _.convertGrade(_.findMetadata('gradeMax', item)[0].text);

    if (from === 'a' || to === 'a') {
      return ['higher education'];
    }

    return ['school'];
  },
  // LOM: primary | secondary | teacher | assessment | question | concept | assignment
  useType: function useType(item) {
    if (_.isParentAddOn(item)) {
      return 'secondary';
    }

    if (_.isTeacherPresentation(item)) {
      return 'teacher';
    }

    if (_.isStudentSubmissionRequired(item)) {
      return 'assignment';
    }

    if (_.isAssessment(item)) {
      return 'assessment';
    }

    return 'primary';
  },
  // LOM: private | shared | public
  visibility: function visibility(item) {
    var role = _.accessPermission(item);

    switch (role) {
      case 'admin':
        return 'private';
      default:
        return 'public';
    }
  },
  findMetadata: function findMetadata(key, item) {
    // If namespace does not exist, return an empty object for when attempting to chain a property.
    if (!item || !item.hasOwnProperty('rawMetadata')) {
      return NOOP_ARRAY;
    }

    var results = item.rawMetadata.filter(function (metadata) {
      return metadata.key === key;
    });

    return results.length > 0 ? results : NOOP_ARRAY;
  },
  shallowClone: function shallowClone(item) {
    return Object.assign({}, item, {
      children: [],
      rawMetadata: []
    });
  }
};

module.exports = _;