'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var fs = require('fs');
var path = require('path');
var archiver = require('archiver');
var getNested = require('lodash/get');
var TEMP_DIR = path.join(__dirname, 'tmp');
var guid = require('./helpers').guid;

/**
 * File builder
 */

var Build = function () {
  /**
   * Build out file(s) from object of Transform
   * @param {object} input
   */
  function Build(input) {
    _classCallCheck(this, Build);

    this._files = [];
    this.ensureTempDir();
    this.createFolder(input);
  }

  _createClass(Build, [{
    key: 'ensureTempDir',
    value: function ensureTempDir() {
      this.ensureDir(this.tmp);
    }
  }, {
    key: 'ensureDir',
    value: function ensureDir(dirname) {
      try {
        fs.mkdirSync(dirname);
      } catch (err) {
        if (err.code !== 'EEXIST') throw err;
      }
    }
  }, {
    key: 'ensurePathToFile',
    value: function ensurePathToFile(filename) {
      var _this = this;

      var basepath = path.join(this.folder);
      var dirnames = filename.split('/');
      var file = dirnames.pop();

      dirnames.forEach(function (dirname) {
        basepath = basepath + '/' + dirname;
        _this.ensureDir(basepath);
      });

      return basepath + '/' + file;
    }
  }, {
    key: 'createFolder',
    value: function createFolder(input) {
      var manifestXML = input['imsmanifest.xml'];
      var cartridgeName = void 0;

      if (manifestXML) {
        cartridgeName = getNested(manifestXML, 'raw.manifest.metadata.lomm:lom.lomm:general.lomm:identifier.lomm:entry');
      }

      this.cartridge = cartridgeName || guid();
      this.folder = this.cartridge;

      this.ensureDir(this.folder);

      for (var filename in input) {
        var scope = input[filename];

        this.createFile(filename, scope.xml);
      }
    }
  }, {
    key: 'createFile',
    value: function createFile(filename, data) {
      var filepath = this.ensurePathToFile(filename);

      fs.writeFileSync(filepath, data);

      this._files.push(filename);
    }
  }, {
    key: 'compress',
    value: function compress() {
      var _this2 = this;

      this.zip = this.cartridge;
      this.file = this.zip;

      var output = fs.createWriteStream(this.file);
      var archive = archiver('zip', { zlib: { level: 9 } });

      archive.on('warning', function (err) {
        if (err.code !== 'ENOENT') throw err;
      });
      archive.on('error', function (err) {
        throw err;
      });
      archive.pipe(output);
      archive.directory(this.folder, false);

      return archive.finalize().then(function () {
        return {
          path: _this2.file,
          filename: _this2.zip,
          name: _this2.cartridge,
          stream: fs.createReadStream(_this2.file)
        };
      }).catch(function (err) {
        throw err;
      });
    }

    /**
     * Remove a single file.
     */

  }, {
    key: 'singleFileCleanup',
    value: function singleFileCleanup(file) {
      if (fs.existsSync(file)) {
        fs.unlinkSync(file);
      }
    }

    /**
     * Provide a directory and this will delete everything contained in it.
     */

  }, {
    key: 'recursiveCleanup',
    value: function recursiveCleanup(folder) {
      var _this3 = this;

      var filepath = void 0;

      if (fs.existsSync(folder)) {
        fs.readdirSync(folder).forEach(function (file) {
          filepath = path.join(folder, file);
          if (fs.lstatSync(filepath).isDirectory()) {
            _this3.recursiveCleanup(filepath);
          } else {
            _this3.singleFileCleanup(filepath);
          }
        });
        fs.rmdirSync(folder);
      }
    }

    /**
     * Removes files and folder, but not archive.
     */

  }, {
    key: 'cleanupFilesAndFolder',
    value: function cleanupFilesAndFolder() {
      this.recursiveCleanup(this.folder);
    }

    /**
     * Removes all files created during the build process.
     */

  }, {
    key: 'cleanup',
    value: function cleanup() {
      this.cleanupFilesAndFolder();
      this.singleFileCleanup(this.file);
    }

    /**
     * Handle sending the zip file
     * @param {object} res
     */

  }, {
    key: 'pipe',
    value: function pipe(res) {
      var _this4 = this;

      return this.compress().then(function (file) {
        var stream = fs.createReadStream(file.path);

        res.writeHead(200, {
          'Content-Disposition': 'attachment; filename=' + file.filename
        });

        stream.on('end', function () {
          _this4.cleanup();
        });

        stream.pipe(res);
      });
    }
  }, {
    key: 'tmp',
    get: function get() {
      return TEMP_DIR;
    }

    /**
    * Return the name of the Common Cartridge directory.
    */

  }, {
    key: 'cartridge',
    get: function get() {
      return this._cartridge;
    },
    set: function set(name) {
      this._cartridge = name;
    }

    /**
     * Return the path to contents of temporary Common Cartridge directory.
     */

  }, {
    key: 'folder',
    get: function get() {
      return this._folder;
    },
    set: function set(name) {
      this._folder = path.join(this.tmp, name);
    }

    /**
     * Return list (manifest) of all files created.
     */

  }, {
    key: 'files',
    get: function get() {
      return this._files;
    }

    /**
     * Return the zip file with extension.
     */

  }, {
    key: 'zip',
    get: function get() {
      return this._zip;
    },
    set: function set(name) {
      this._zip = name + '.imscc';
    }

    /**
     * Return the path to the zip file.
     */

  }, {
    key: 'file',
    get: function get() {
      return this._file;
    },
    set: function set(zip) {
      this._file = path.join(this.tmp, zip);
    }
  }]);

  return Build;
}();

module.exports = Build;